/// Return a list of nodes in a order of breath search. (The graph must be cycle free.) 
List<T> breathSearch<T>(T root, Iterable<T> getChildren(T node)) {
  if (root == null) {
    throw ArgumentError.notNull("root");
  }
  if (getChildren == null) {
    throw ArgumentError.notNull("getChildren");
  }

  var result = List.of([root]);
  var children = getChildren(root);

  if (children == null || children.isEmpty) {
    return result;
  }

  var iterator = children.iterator;

  while (true) {
    var allChildren = Set<T>();
    while (iterator != null && iterator.moveNext()) {
      var current = iterator.current;

      if (result.contains(current)) {
        continue;
      }

      result.add(current);

      children = getChildren(current);
      if (children == null) {
        continue;
      }

      allChildren.addAll(children);
    }

    if (allChildren.isEmpty) {
      break;
    }

    iterator = allChildren.iterator;
  }

  return result;
}
