import 'package:riedev_graph/graph.dart';

/// Creates a new graph with the given [root] and the specify [getChildren] method.
/// The edges will created through the [createEdge] factory method.
Graph<TNode, TEdge> createNewGraph<TNode, TEdge extends Edge<TNode>>(
    TNode root,
    Iterable<TNode> getChildren(TNode node),
    TEdge createEdge(TNode source, TNode target)) {
  if (root == null) {
    throw ArgumentError.notNull("root");
  }
  if (getChildren == null) {
    throw ArgumentError.notNull("getChildren");
  }
  if (createEdge == null) {
    throw ArgumentError.notNull("createEdge");
  }

  var graph = Graph<TNode, TEdge>(false);

  var visitedNodes = Set<TNode>();
  var iterators = List<Iterator<TNode>>();

  iterators.add([root].iterator);

  while (iterators.isNotEmpty) {
    var iterator = iterators.last;
    while (iterator != null && iterator.moveNext()) {
      var current = iterator.current;

      if (visitedNodes.contains(current)) {
        continue;
      }

      visitedNodes.add(current);

      if (!graph.Nodes.contains(current)) {
        graph.addNode(current);
      }

      var children = getChildren(current);
      if (children == null) {
        continue;
      }

      for (var child in children) {
        graph.addEdge(createEdge(current, child));
      }

      iterator = children.iterator;
      iterators.add(iterator);
    }
    iterators.remove(iterator);
  }

  return graph;
}
