import 'dart:math';

/// Return the level for every visited node inside the graph. (The graph must be cycle free.)
/// The level is definied as the maximum step number (one edge == one step) to the root node. 
Map<T, int> dependencySearch<T>(T root, Iterable<T> getChildren(T node)) {
  if (root == null) {
    throw ArgumentError.notNull("root");
  }
  if (getChildren == null) {
    throw ArgumentError.notNull("getChildren");
  }

  var result = Map.of({root: 0});
  var children = getChildren(root);

  if (children == null || children.isEmpty) {
    return result;
  }

  var level = 1;
  var iterator = children.iterator;

  while (true) {
    var allChildren = Set<T>();
    while (iterator != null && iterator.moveNext()) {
      var current = iterator.current;

      if (result.containsKey(current)) {
        result[current] = max(result[current], level);
      }

      result[current] = level;

      children = getChildren(current);
      if (children == null) {
        continue;
      }

      allChildren.addAll(children);
    }

    level++;
    if (allChildren.isEmpty) {
      break;
    }

    iterator = allChildren.iterator;
  }

  return result;
}
